import type { ChangeEvent } from "react";
import { useState } from "react";
import "./App.css";
import Person from "./Person/Person";
import IPerson from "./interfaces/IPerson";

type State = (IPerson & {
  hobby: string;
})[];

const App = () => {
  const [people, setPeople] = useState<State>([
    {
      name: "John",
      age: 33,
      hobby: "video games",
      id: "j11",
    },
    {
      name: "Jane",
      age: 28,
      hobby: "video games",
      id: "j12",
    },
    {
      name: "Dmitry",
      age: 33,
      hobby: "video games",
      id: "j13",
    },
    {
      name: "Kamisama",
      age: 28,
      hobby: "video games",
      id: "j14",
    },
  ]);

  const [isShowPeople, setIsShowPeople] = useState(true);

  const increaseAge = (id: string): void => {
    const index = people.findIndex((person) => person.id === id);
    const person = { ...people[index] };
    person.age++;
    const copyPeople = [...people];
    copyPeople[index] = person;

    setPeople(copyPeople);
  };

  const changeNameHandler = (
    e: ChangeEvent<HTMLInputElement>,
    index: number
  ) => {
    const copyPeople = structuredClone(people);
    copyPeople[index].name = e.target.value;

    setPeople(copyPeople);
  };

  const toggleIsShowPeople = (): void => {
    setIsShowPeople(!isShowPeople);
  };

  const removePerson = (index: number): void => {
    const copyPeople = structuredClone(people);
    copyPeople.splice(index, 1);
    setPeople(copyPeople);
    // setPeople((currentState) => {
    //   console.log(currentState);

    //   return copyPeople;
    // });
  };
  console.log("in App", people);
  return (
    <>
      <div className="App">
        <button onClick={toggleIsShowPeople}>
          {isShowPeople ? "hide" : "show"}
        </button>
        {isShowPeople &&
          people.map((person, index) => {
            return (
              <Person
                key={person.id}
                name={person.name}
                age={person.age}
                onClickHeader={() => {
                  increaseAge(person.id);
                }}
                onChangeHandler={(e) => {
                  changeNameHandler(e, index);
                }}
                onRemoveHandler={() => {
                  removePerson(index);
                }}
              >
                hobby:{person.hobby}
              </Person>
            );
          })}
      </div>
    </>
  );
};

export default App;
